<?php

namespace Teknisa\MAN\Services;

use Teknisa\MAN\Factories\RelatoriosRNFFactory;

class RelatoriosRNFService {
    protected $relatoriosRNFFactory;
    protected $inOrdem;
    protected $cdsRelatorio;
    protected $stNovoItem;

    public function __construct(RelatoriosRNFFactory $relatoriosRNFFactory) {
        $this->relatoriosRNFFactory = $relatoriosRNFFactory;
    }

    public function find($params) {
        $result = [];
        $paramsRelatorio = [
            'P_CDFILIAL'   => $params['P_CDFILIAL'],
            'P_DTINICIAL'  => $params['P_DTINICIAL'],
            'P_DTFINAL'    => $params['P_DTFINAL'],
            'P_CDOPERADOR' => $params['P_CDOPERADOR']
        ];
        
        if($params['IDRELATORIO'] == 0) {
            // Relatório Validações RNF
            $result = $this->relatoriosRNFFactory->findValidacoesRNF($paramsRelatorio);
            
            // $result = $this->formataDados($result);
            
        } else if($params['IDRELATORIO'] == 1) {
            $paramsRelatorio['P_IDSTATUS'] = $params['P_IDSTATUS'];
            
            // Relatório Resumo de Notas Fiscais
            $result = $this->relatoriosRNFFactory->findResumoNotasFiscais($paramsRelatorio);
            
        } else if($params['IDRELATORIO'] == 2) {
            // Relatório Detalhamento de RNF
            $result = $this->relatoriosRNFFactory->findDetalhamentoRNF($paramsRelatorio);
            
        } else if($params['IDRELATORIO'] == 3) {
            // Relatório Validações Baseadas em RNF
            $result = $this->getRelVdRNF($paramsRelatorio);
    
        } else if ($params['IDRELATORIO'] == 4) {
            $paramsRelatorio['P_IDSTATUS'] = $params['P_IDSTATUS'];
            $result = $this->relatoriosRNFFactory->findConferenciaRNF($paramsRelatorio);
        }
        
        return $result;
    }
    
    private function formataDados($array) {
        $result = [];
        
        foreach($array as $i => $arrayExterno) {
            foreach($arrayExterno as $j => $arrayInterno) {
                if(isset($arrayExterno)) {
                    $result[]['DSVALIDACAO'] = $arrayInterno['DSVALIDACAO'];
                }
            }
        }
        
        return $result;
    }

    private function getRelVdRNF($params) {
        try {
            $this->inOrdem = 0;
            $i = 0;
            
            // roda query
            $adqRelFil = $this->relatoriosRNFFactory->adqRelFil($params);
            
            if(!empty($adqRelFil)) {
                foreach($adqRelFil as $relFil) {
                    $this->validacaoGeral($relFil, $params, $i);

                    // roda query
                    $adqRelBody = $this->relatoriosRNFFactory->adqRelBody($params, $relFil['CDFILIAL']);

                    if(!empty($adqRelBody)) {
                        foreach($adqRelBody as $relBody) {
                            // roda query
                            $adqRelItens = $this->relatoriosRNFFactory->adqRelItens($relFil, $relBody);

                            if(!empty($adqRelItens)) {
                                foreach($adqRelItens as $relItens) {
                                    $this->stNovoItem = 'S';
                                    $this->cdsRelatorio[$i]['NRORDEM'] = $this->seqOrdem();

                                    $this->infItem($relBody, $relFil, $relItens, $i);
                                    
                                    // roda query
                                    $adqCodExtProd = $this->relatoriosRNFFactory->adqCodExtProd($relItens['CDARVPROD']);
                                    
                                    if(!empty($adqCodExtProd)) {
                                        $this->cdsRelatorio[$i]['NRORDEM'] = $this->seqOrdem();

                                        $this->infItem($relBody, $relFil, $relItens, $i);

                                        $this->cdsRelatorio[$i]['DSVALIDACAO'] = $adqCodExtProd[0]['VDPRODUTO'];
                                    }

                                    // roda query
                                    $adqRelNCM = $this->relatoriosRNFFactory->adqRelNCM($relItens['CDARVPROD']);

                                    if(!empty($adqRelNCM)) {
                                        $this->cdsRelatorio[$i]['NRORDEM'] = $this->seqOrdem();

                                        $this->infItem($relBody, $relFil, $relItens, $i);

                                        $this->cdsRelatorio[$i]['DSVALIDACAO'] = $adqRelNCM['DSVALIDACAO'];
                                    }

                                    // roda query
                                    $adqTamNCM = $this->relatoriosRNFFactory->adqTamNCM($relItens['CDARVPROD']);

                                    if(!empty($adqTamNCM)) {
                                        $this->cdsRelatorio[$i]['NRORDEM'] = $this->seqOrdem();

                                        $this->infItem($relBody, $relFil, $relItens, $i);

                                        $this->cdsRelatorio[$i]['DSVALIDACAO'] = $adqTamNCM['DSVALIDACAO'];
                                    }
                                }
                            }
                        }
                    }
                    $i++;
                }
            }

            $result = $this->cdsRelatorio;
            return $result;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    private function validacaoGeral($relFil, $params, $i) {
        $this->checaPeriodo($relFil, $params, $i);
        $this->checaCodExtFil($relFil, $i);
        $this->checaTamCodFil($relFil, $i);
        $this->checaCodExtEmpr($relFil, $i);
        $this->checaCodConvFilial($relFil, $i);
        $this->checaClienPadrao($relFil, $i);
        $this->checaCodExtClienPadrao($relFil, $i);
        $this->checaTipoEmisCaixa($relFil, $i);
        $this->checaTamCodExtEmpr($relFil, $i);
        $this->checaUltFechamento($relFil, $params, $i);
    }

    private function infItem($adqRelBody, $adqRelFil, $adqRelItens, $i) {
        $this->infFilial($adqRelFil, $i);
        
        $this->cdsRelatorio[$i]['NOTA']    = $adqRelBody['NRNOTAFISC'];
        $this->cdsRelatorio[$i]['CDSERIE'] = $adqRelBody['CDSERIE'];
        $this->cdsRelatorio[$i]['DTNOTA']  = $adqRelBody['DTENTRVENDA'];
        $this->cdsRelatorio[$i]['CPFCNPJ'] = $adqRelFil['CDEXCLIENTE'];
        $this->cdsRelatorio[$i]['CDITEM']  = $adqRelItens['CDARVPROD'];
        $this->cdsRelatorio[$i]['NMITEM']  = $adqRelItens['NMPRODUTO'];

        if(!empty($adqRelItens)) {
            $this->cdsRelatorio[$i]['DSNCM'] = substr($adqRelItens['CDNCM'], 1, 4) . '.' . 
                                            substr($adqRelItens['CDNCM'], 5, 2) . '.' . 
                                            substr($adqRelItens['CDNCM'], 7, 2);
        }

        $this->cdsRelatorio[$i]['VRPRODUTOS'] = $adqRelItens['QTDEITEM'] * $adqRelItens['VRUNIT'];
        $this->cdsRelatorio[$i]['VROPERACAO'] = $adqRelItens['VRITEM'];
        $this->cdsRelatorio[$i]['TIPOOPERAC'] = $adqRelBody['CODOPER'];

        // roda query
        $adqRelImposto = $this->relatoriosRNFFactory->adqRelImposto($adqRelBody, $adqRelFil, $adqRelItens);
        
        if(!empty($adqRelImposto)) {
            $this->cdsRelatorio[$i]['VLRICMS']   = $adqRelImposto[0]['VRIMPOPRODIT'];
            $this->cdsRelatorio[$i]['VLRPIS']    = $adqRelImposto[0]['VRIMPPIS'];
            $this->cdsRelatorio[$i]['VLRCOFINS'] = $adqRelImposto[0]['VRIMPCOFINS'];
            $this->cdsRelatorio[$i]['CDCFOP']    = $adqRelImposto[0]['CDCFOPPROD'];
        }

        // roda query
        $adqRelTipoDoc = $this->relatoriosRNFFactory->adqRelTipoDoc($adqRelItens);

        if(empty($adqRelTipoDoc)) {
            $this->cdsRelatorio[$i]['DSTIPODOC'] = "";

            if($this->stNovoItem == 'S') {
                $this->stNovoItem = 'N';
                $this->cdsRelatorio[$i]['DSVALIDACAO'] = "TIPO DE DOCUMENTO NAO CADASTRADO PARA O CFOP " . $adqRelItens['CDCFOPPROD'] . 
                                                     ", NCM " . $adqRelItens['CDNCM'] . " E CST (ICMS) " . $adqRelItens['CDCSTPRODI'];
            }
        } else {
            $this->cdsRelatorio[$i]['DSTIPODOC'] = $adqRelTipoDoc[0]['CDTPDOC'];
        }
    }

    private function seqOrdem() {
        return $this->inOrdem++;
    }

    private function infFilial($adqRelFil, $i) {
        $this->cdsRelatorio[$i]['CDFILIALEXT']  = $adqRelFil['CDFILIALEXT'];
        $this->cdsRelatorio[$i]['NMFILIAL']     = $adqRelFil['NMFILIAL'];
        $this->cdsRelatorio[$i]['CDCLIENTE']    = $adqRelFil['CDCLIENTE'];
        $this->cdsRelatorio[$i]['NMRAZSOCCLIE'] = $adqRelFil['NMRAZSOCCLIE'];
    }

    private function checaPeriodo($relFil, $params, $i) {
        // roda query
        $adqRelPeriodo = $this->relatoriosRNFFactory->adqRelPeriodo($relFil, $params);
        
        if(!empty($adqRelPeriodo)) {
            foreach($adqRelPeriodo as $relPeriodo) {
                $this->cdsRelatorio[$i]['DSVALIDACAO'] = $relPeriodo['DSVALIDACAO'];
                
                $this->infFilial($relFil, $i);
                $this->cdsRelatorio[$i]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaCodExtFil($relFil, $i) {
        // roda query
        $adqRelCodExtFil = $this->relatoriosRNFFactory->adqRelCodExtFil($relFil);

        if(!empty($adqRelCodExtFil)) {
            foreach($adqRelCodExtFil as $relCodExtFil) {
                $this->infFilial($relFil, $i);
                $this->cdsRelatorio[$i]['DSVALIDACAO'] = $relCodExtFil['FILIALVD'];                
                $this->cdsRelatorio[$i]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaTamCodFil($relFil, $i) {
        // roda query
        $adqRelTamCodFil = $this->relatoriosRNFFactory->adqRelTamCodFil($relFil);

        if(!empty($adqRelTamCodFil)) {
            foreach($adqRelTamCodFil as $relTamCodFil) {
                $this->infFilial($relFil, $i);
                $this->cdsRelatorio[]['DSVALIDACAO'] = $relTamCodFil['FILIALVD'];                
                $this->cdsRelatorio[]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaCodExtEmpr($relFil, $i) {
        // roda query
        $adqRelCodExtEmpr = $this->relatoriosRNFFactory->adqRelCodExtEmpr($relFil);

        if(!empty($adqRelCodExtEmpr)) {
            foreach($adqRelCodExtEmpr as $relCodExtEmpr) {
                $this->infFilial($relFil, $i);
                $this->cdsRelatorio[$i]['DSVALIDACAO'] = $relCodExtEmpr['DSVALIDACAO'];                
                $this->cdsRelatorio[$i]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaCodConvFilial($relFil, $i) {
        // roda query
        $adqRelConvFil = $this->relatoriosRNFFactory->adqRelConvFil($relFil);

        if(!empty($adqRelConvFil)) {
            foreach($adqRelConvFil as $relConvFil) {
                $this->infFilial($relFil, $i);
                $this->cdsRelatorio[$i]['DSVALIDACAO'] = $relConvFil['VDFILIAL'];                
                $this->cdsRelatorio[$i]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaClienPadrao($relFil, $i) {
        // roda query
        $adqRelClienPadrao = $this->relatoriosRNFFactory->adqRelClienPadrao($relFil);

        if(!empty($adqRelClienPadrao)) {
            foreach($adqRelClienPadrao as $relClienPadrao) {
                $this->infFilial($relFil, $i);
                $this->cdsRelatorio[$i]['DSVALIDACAO'] = $relClienPadrao['VDFILIAL'];                
                $this->cdsRelatorio[$i]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaCodExtClienPadrao($relFil, $i) {
        // roda query
        $adqRelCodExtClien = $this->relatoriosRNFFactory->adqRelCodExtClien($relFil);

        if(!empty($adqRelCodExtClien)) {
            foreach($adqRelCodExtClien as $relCodExtClien) {
                $this->infFilial($relFil, $i);
                $this->cdsRelatorio[$i]['DSVALIDACAO'] = $relCodExtClien['VDFILIAL'];                
                $this->cdsRelatorio[$i]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaTipoEmisCaixa($relFil, $i) {
        // roda query
        $adqRelTipoEmis = $this->relatoriosRNFFactory->adqRelTipoEmis($relFil);

        if(!empty($adqRelTipoEmis)) {
            foreach($adqRelTipoEmis as $relTipoEmis) {
                $this->infFilial($relFil, $i);
                $this->cdsRelatorio[$i]['DSVALIDACAO'] = $relTipoEmis['VDFILIAL'];                
                $this->cdsRelatorio[$i]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaTamCodExtEmpr($relFil, $i) {
        // roda query
        $adqRelTamCodExtEmpr = $this->relatoriosRNFFactory->adqRelTamCodExtEmpr($relFil);

        if(!empty($adqRelTamCodExtEmpr)) {
            foreach($adqRelTamCodExtEmpr as $relTamCodExtEmpr) {
                $this->infFilial($relFil, $i);
                $this->cdsRelatorio[$i]['DSVALIDACAO'] = $relTamCodExtEmpr['VDEMPRESA'];                
                $this->cdsRelatorio[$i]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaUltFechamento($relFil, $params, $i) {
        // roda query
        $adqMaxDtAbert = $this->relatoriosRNFFactory->adqMaxDtAbert($relFil, $params);

        if(!empty($adqMaxDtAbert)) {
            foreach($adqMaxDtAbert as $maxDtAbert) {
                // roda query
                $adqChecaFechamento = $this->relatoriosRNFFactory->adqChecaFechamento($maxDtAbert);

                if(!empty($adqChecaFechamento)) {
                    $this->infFilial($relFil, $i);
                    $this->cdsRelatorio[$i]['DSVALIDACAO'] = $adqChecaFechamento[0]['DSVALIDACAO'];                
                    $this->cdsRelatorio[$i]['NRORDEM'] = $this->seqOrdem();
                }
            }
        }
    }
}